import torch

from deepview_profile.exceptions import AnalysisError
from deepview_profile.models.analysis import PerformanceLimits


def to_trainable_model(parse_tree, class_name):
    try:
        executable = compile(parse_tree, '<string>', 'exec')
        scope = {}
        exec(executable, scope, scope)
        model = scope[class_name]().to(torch.device('cuda'))
        model.train()
        return model
    except Exception as ex:
        raise AnalysisError(str(ex), type(ex))


def get_performance_limits(memory_info, throughput_info):
    max_capacity_batch_size = memory_info.usage_model_mb.inverse(
        memory_info.max_capacity_mb)
    max_capacity_throughput = (
        max_capacity_batch_size /
        throughput_info.runtime_model_ms.evaluate(max_capacity_batch_size) *
        1000
    )
    max_throughput_batch_size = throughput_info.batch_from_throughput(
        throughput_info.max_throughput)

    thpt_limits = (max_throughput_batch_size, throughput_info.max_throughput)
    mem_limits = (max_capacity_batch_size, max_capacity_throughput)

    limits = min(thpt_limits, mem_limits, key=lambda tup: tup[0])

    return PerformanceLimits(
        max_batch_size=limits[0],
        throughput_limit=limits[1],
    )



from scipy.stats import gaussian_kde
import numpy as np
import os
import logging
from deepview_profile.pytorch_profiler_log_reader import (
    get_first_last_step,
    get_bucket_sizes,
    get_ddp_forward_backward_times,
)
import time
from torch.profiler import profile, schedule, ProfilerActivity
import torch.distributed as dist
from torch.nn.parallel import DistributedDataParallel as DDP
import subprocess

logger = logging.getLogger(__name__)

FILENAME = "pytorch_profiler.json"
RANK = 0
WORLD_SIZE = 1
DEFAULT_BUCKET_SIZE = 25


def setup(rank, world_size):
    os.environ["MASTER_ADDR"] = "localhost"
    os.environ["MASTER_PORT"] = "12345"
    dist.init_process_group("nccl", rank=rank, world_size=world_size)


def cleanup():
    dist.destroy_process_group()


def _bucket_estimate_max_expected(bucket_times, ngpu):
    m = 1000

    np_samples = np.array(bucket_times)

    kde_samples = gaussian_kde(np_samples)

    z_arr = []
    for _ in range(m):
        num_resamples = kde_samples.resample(ngpu)

        z_arr.append(np.max(num_resamples))

    expected_max = np.mean(z_arr)

    return expected_max


def _bucket_comp_times(path_to_file):
    data_matrix = []
    first_step, last_step = get_first_last_step(path_to_file)
    NUM_STEPS = 25
    forward_time_acc = 0
    for step in range(first_step + 1, first_step + NUM_STEPS + 1):
        fw_time, bucket_comp_times = get_ddp_forward_backward_times(path_to_file, step)
        forward_time_acc += fw_time
        """
        storing as:
        [bucket_0 time1, bucket_1 time1, ... , bucket_n time1]
        [bucket_0 time2, bucket_1 time2, ... , bucket_n time2]
        ...
        """
        data_matrix.append(bucket_comp_times)
    # convert to numpy and transpose
    data_numpy = np.array(data_matrix)
    """
    store as :
    [bucket_0 time1, bucket_0 time2, ...., bucket_0 time n]
    [bucket_1 time1, bucket_1 time2, ...., bucket_1 time n]
    """
    data_transpose = np.transpose(data_numpy)

    return forward_time_acc / NUM_STEPS, data_transpose


def _bucket_expected_max(bucket_times, ngpus):
    expected_max_arr = []
    for samples in bucket_times:
        expected_max = _bucket_estimate_max_expected(samples, ngpus)
        expected_max_arr.append(expected_max)

    return expected_max_arr


def _trace_handler(p):
    p.export_chrome_trace(FILENAME)


def run_profiler(model_provider, input_provider, iteration_provider):
    setup(RANK, WORLD_SIZE)

    model = model_provider()
    inputs = input_provider()
    ddp_model = DDP(model, device_ids=[RANK], bucket_cap_mb=DEFAULT_BUCKET_SIZE)
    iteration = iteration_provider(ddp_model)
    # warmup for 30 secs
    start = time.time()
    elapsed = 0

    while elapsed < 30:
        for _ in range(100):
            iteration(*inputs)
        elapsed = time.time() - start

    skip_first = 10
    wait = 5
    warmup = 10
    active = 30
    totalIterations = skip_first + wait + warmup + active
    deepviewSchedule = schedule(
        skip_first=skip_first, wait=wait, warmup=warmup, active=active, repeat=1
    )

    with profile(
        activities=[ProfilerActivity.CPU, ProfilerActivity.CUDA],
        schedule=deepviewSchedule,
        on_trace_ready=_trace_handler,
    ) as p:
        for _ in range(totalIterations):
            iteration(*inputs)
            p.step()

    cleanup()


def ddp_analysis(model_provider, input_provider, iteration_provider):
    run_profiler(model_provider, input_provider, iteration_provider)

    path_to_file = os.path.join(os.getcwd(), FILENAME)

    fw_avg_msec, bucket_comp_times = _bucket_comp_times(path_to_file)
    bucket_sizes_arr = get_bucket_sizes(model_provider(), DEFAULT_BUCKET_SIZE)

    expected_max_2gpus = _bucket_expected_max(bucket_comp_times, 2)
    expected_max_4gpus = _bucket_expected_max(bucket_comp_times, 4)

    jsonFormat = {
        "forward_time_ms": fw_avg_msec,
        "bucket_sizes": bucket_sizes_arr,
        "expected_computation_times": [
            {"ngpus": 2, "expected_max_times": expected_max_2gpus},
            {"ngpus": 4, "expected_max_times": expected_max_4gpus},
        ],
    }

    subprocess.run(["rm", "-f", os.path.join(os.getcwd(), FILENAME)])
    return jsonFormat



import logging
from concurrent.futures import ThreadPoolExecutor

from deepview_profile.analysis.request_manager import AnalysisRequestManager
from deepview_profile.io.connection_acceptor import ConnectionAcceptor
from deepview_profile.io.connection_manager import ConnectionManager
from deepview_profile.protocol.message_handler import MessageHandler
from deepview_profile.protocol.message_sender import MessageSender

logger = logging.getLogger(__name__)


class SkylineServer:
    def __init__(self, host, port):
        self._requested_host = host
        # This is the port the user specified on the command line (it can be 0)
        self._requested_port = port
        self._connection_acceptor = ConnectionAcceptor(
            self._requested_host,
            self._requested_port,
            self._on_new_connection,
        )
        self._connection_manager = ConnectionManager(
            self._on_message,
            self._on_connection_closed,
        )
        self._message_sender = MessageSender(self._connection_manager)
        self._analysis_request_manager = AnalysisRequestManager(
            self._submit_work,
            self._message_sender,
            self._connection_manager,
        )
        self._message_handler = MessageHandler(
            self._connection_manager,
            self._message_sender,
            self._analysis_request_manager,
        )
        self._main_executor = ThreadPoolExecutor(max_workers=1)

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stop()

    def start(self):
        self._analysis_request_manager.start()
        self._connection_acceptor.start()
        logger.debug("DeepView server has started.")

    def stop(self):
        def shutdown():
            self._connection_acceptor.stop()
            self._connection_manager.stop()

        self._analysis_request_manager.stop()
        self._main_executor.submit(shutdown).result()
        self._main_executor.shutdown()
        logger.debug("DeepView server has shut down.")

    @property
    def listening_on(self):
        return (self._connection_acceptor.host, self._connection_acceptor.port)

    def _on_message(self, data, address):
        print("on_message:", data, address)
        # Do not call directly - called by a connection
        self._main_executor.submit(
            self._message_handler.handle_message,
            data,
            address,
        )

    def _on_new_connection(self, socket, address):
        print("on_new_connection", socket, address)
        # Do not call directly - called by _connection_acceptor
        self._main_executor.submit(
            self._connection_manager.register_connection,
            socket,
            address,
        )

    def _on_connection_closed(self, address):
        # Do not call directly - called by a connection when it is closed
        self._main_executor.submit(
            self._connection_manager.remove_connection,
            address,
        )

    def _submit_work(self, func, *args, **kwargs):
        print("submit_work", func)
        # print("submit_work args:", args)
        logger.debug("submit_work args:", args)
        print("submit_work kwargs:", kwargs)
        # Do not call directly - called by another thread to submit work
        # onto the main executor
        self._main_executor.submit(func, *args, **kwargs)



import collections
import gc

import torch

from deepview_profile.tracking.backward_interceptor import BackwardInterceptor
from deepview_profile.tracking.base import TrackerBase
from deepview_profile.tracking.call_stack import CallStack
from deepview_profile.tracking.callable_tracker import CallableTracker
from deepview_profile.tracking.utils import remove_dunder
from deepview_profile.user_code_utils import user_code_environment

OperationContext = collections.namedtuple(
    'OperationContext',
    ['operation_name', 'stack'],
)

ActivationEntry = collections.namedtuple(
    'ActivationEntry',
    ['operation_name', 'stack', 'size_bytes'],
)


class ActivationsTracker:
    def __init__(self, project_root):
        self._activations = []
        self._project_root = project_root

    def track_memory_usage(self, iteration, input_provider, user_code_path):
        # 1. Run the forward pass of the model with the given inputs. We keep
        #    track of all the operations that contribute to the autograd graph.
        model_output, grad_function_contexts = \
            self._get_grad_function_contexts(
                iteration, input_provider, user_code_path)

        # 2. Traverse the autograd graph and get a topological ordering. Filter
        #    the function contexts by the gradient functions in our topological
        #    ordering.
        gradient_functions_topo_order, grad_function_contexts = \
            self._extract_relevant_gradient_functions(
                model_output, grad_function_contexts)

        # 3. Associate activation sizes with each gradient function by
        #    effectively "freeing" them one after the other and tracking the
        #    change in overall memory allocations.

        # NOTE: We reverse the list here to be able to pop from it in
        #       topological order.
        gradient_functions_topo_order.reverse()
        del model_output
        gc.collect()

        while len(gradient_functions_topo_order) > 0:
            grad_fn = gradient_functions_topo_order.pop()
            context = grad_function_contexts[grad_fn]
            del grad_function_contexts[grad_fn]

            mem_before = torch.cuda.memory_allocated()
            del grad_fn
            gc.collect()
            mem_after = torch.cuda.memory_allocated()
            delta = mem_after - mem_before
            self._activations.append(ActivationEntry(
                *context,
                size_bytes=-delta,
            ))

    def populate_report(self, builder):
        for entry in self._activations:
            builder.add_activation_entry(
                operation_name=remove_dunder(entry.operation_name),
                size_bytes=entry.size_bytes,
                stack_context=entry.stack,
            )

    def populate_breakdown(self, builder):
        # The HierarchicalReportBuilder uses the same activation entry API as
        # the MemoryReportBuilder
        self.populate_report(builder)

    def _get_grad_function_contexts(
            self, iteration, input_provider, user_code_path):
        grad_function_tracker = GradFunctionTracker(self._project_root)
        backward_interceptor = BackwardInterceptor()
        with grad_function_tracker.track(), \
                backward_interceptor.intercept(), \
                user_code_environment(user_code_path, self._project_root):
            iteration(*input_provider())
        return (
            backward_interceptor.backward_root,
            grad_function_tracker.grad_function_contexts,
        )

    def _extract_relevant_gradient_functions(
            self, model_output, grad_function_contexts):
        # 1. Get the gradient functions associated with the model output in
        #    topological order
        gradient_functions = \
            _extract_gradient_functions_in_topological_order(model_output)

        # 2. Filter the gradient functions: we only want to keep the ones we
        #    know about
        relevant_grad_fns = []
        relevant_contexts = {}
        for grad_fn in gradient_functions:
            if grad_fn not in grad_function_contexts:
                continue
            relevant_grad_fns.append(grad_fn)
            relevant_contexts[grad_fn] = grad_function_contexts[grad_fn]

        return relevant_grad_fns, relevant_contexts


class GradFunctionTracker(TrackerBase):
    def __init__(self, project_root):
        super().__init__()
        self._callable_tracker = CallableTracker(self._callable_hook_creator)
        self._project_root = project_root
        self.grad_function_contexts = {}
        self._processing_hook = False

    def start_tracking(self):
        super().start_tracking()
        self.grad_function_contexts.clear()
        self._callable_tracker.start_tracking()

    def stop_tracking(self):
        super().stop_tracking()
        self._callable_tracker.stop_tracking()

    def _callable_hook_creator(self, func):
        def hook(*args, **kwargs):
            # NOTE: We use self._processing_hook to handle cases where we have
            #       hooks on nested function calls.
            if self._processing_hook:
                return func(*args, **kwargs)

            self._processing_hook = True
            try:
                retval = func(*args, **kwargs)
            finally:
                self._processing_hook = False

            # Early return for tensor-producing operations that are not
            # involved in the backward pass
            if (not isinstance(retval, torch.Tensor) and
                    not isinstance(retval, tuple) and
                    not isinstance(retval, list)):
                return retval
            if (isinstance(retval, torch.Tensor) and
                    (not retval.is_cuda or retval.grad_fn is None)):
                return retval

            stack = CallStack.from_here(self._project_root, start_from=2)
            if len(stack.frames) == 0:
                return retval

            context = OperationContext(
                operation_name=func.__name__,
                stack=stack,
            )
            self._handle_callable_result(retval, context)
            return retval

        return hook

    def _handle_callable_result(self, retval, context):
        if isinstance(retval, torch.Tensor) and retval.grad_fn is not None:
            self.grad_function_contexts[retval.grad_fn] = context

        elif isinstance(retval, tuple) or isinstance(retval, list):
            for inner_value in retval:
                self._handle_callable_result(inner_value, context)


def _extract_gradient_functions_in_topological_order(model_output):
    """
    Given a model output (Tensor or nested list/tuple of Tensors), build a
    topological ordering of their gradient functions.
    """
    if isinstance(model_output, tuple) or isinstance(model_output, list):
        tensors = _flatten_and_filter_tensors(model_output)
    elif (isinstance(model_output, torch.Tensor) and
          model_output.grad_fn is not None):
        tensors = [model_output]
    else:
        return []

    result = []
    visited = {tensor.grad_fn for tensor in tensors}
    stack = [(grad_fn, 0) for grad_fn in visited]

    while len(stack) > 0:
        grad_fn, visit_count = stack.pop()

        if visit_count != 0:
            result.append(grad_fn)
            continue

        stack.append((grad_fn, 1))

        for fn, _ in grad_fn.next_functions:
            if fn is None or fn in visited:
                continue
            visited.add(fn)
            stack.append((fn, 0))

    result.reverse()
    return result


def _flatten_and_filter_tensors(tensor_iterable):
    flattened = []
    for iterable_element in tensor_iterable:
        if (isinstance(iterable_element, torch.Tensor) and
                iterable_element.grad_fn is not None):
            flattened.append(iterable_element)
        elif (isinstance(iterable_element, tuple) or
              isinstance(iterable_element, list)):
            flattened.extend(_flatten_and_filter_tensors(iterable_element))
    return flattened


import collections


from deepview_profile.tracking.call_stack import CallStack
from deepview_profile.tracking.base import TrackerBase
from deepview_profile.tracking.callable_tracker import CallableTracker
from deepview_profile.tracking.utils import remove_dunder
from deepview_profile.profiler.operation import OperationProfiler

OperationInfo = collections.namedtuple(
    'OperationInfo', ['operation_name', 'stack', 'forward_ms', 'backward_ms'])


class OperationRunTimeTracker(TrackerBase):
    def __init__(self, project_root):
        super().__init__()
        self._callable_tracker = CallableTracker(self._hook_creator)
        self._profiler = OperationProfiler()
        self._project_root = project_root
        self._processing_hook = False

        self.operations = []

    def start_tracking(self):
        super().start_tracking()
        self._callable_tracker.start_tracking()

    def stop_tracking(self):
        super().stop_tracking()
        self._callable_tracker.stop_tracking()

    def populate_report(self, builder):
        for op_info in self.operations:
            builder.add_run_time_entry(
                operation_name=remove_dunder(op_info.operation_name),
                forward_ms=op_info.forward_ms,
                backward_ms=op_info.backward_ms,
                stack_context=op_info.stack,
            )

    def populate_breakdown(self, builder):
        # The HierarchicalBreakdownBuilder uses the same run time entry API as
        # the OperationRunTimeReportBuilder.
        self.populate_report(builder)

    def _hook_creator(self, func):
        def hook(*args, **kwargs):
            # NOTE: We use self._processing_hook to handle cases where we have
            #       hooks on nested function calls.
            if self._processing_hook:
                return func(*args, **kwargs)

            self._processing_hook = True
            try:
                stack = CallStack.from_here(self._project_root, start_from=2)
                if len(stack.frames) == 0:
                    return func(*args, **kwargs)

                forward_ms, backward_ms = self._profiler.measure_operation_ms(
                    func, args, kwargs)
                self.operations.append(OperationInfo(
                    operation_name=func.__name__,
                    stack=stack,
                    forward_ms=forward_ms,
                    backward_ms=backward_ms,
                ))

                # Actually run the hooked function
                return func(*args, **kwargs)
            finally:
                self._processing_hook = False

        return hook



import torch

from deepview_profile.profiler.backward import get_grad_fn, flatten_operation_output


class AutogradEngine:
    """
    Emulates the backward pass for a given model output, for timing purposes.
    """
    def __init__(self, grad_fn_ordering, input_map, initial_inputs):
        self._grad_fn_ordering = grad_fn_ordering
        self._input_holder = {
            fn: [None] * size for fn, size in input_map.items()
        }
        self._input_holder[self._grad_fn_ordering[0]] = initial_inputs

    @classmethod
    def new_from(cls, operation_output, exclude_accumulate_grad=True):
        # Traverse the autograd graph, build input map for each grad_fn and
        # create a topological ordering
        _, initial_grad_fn = get_grad_fn(operation_output)
        if initial_grad_fn is None:
            raise ValueError('No grad_fn available on the operation output.')

        ordering = []
        input_map = {}
        initial_inputs = [
            tensor.detach()
            for tensor in flatten_operation_output(operation_output)
        ]
        input_map[initial_grad_fn] = len(initial_inputs)

        stack = [(initial_grad_fn, 0)]
        visited = {initial_grad_fn}

        # Build a topological ordering
        while len(stack) > 0:
            grad_fn, visit_count = stack.pop()
            if visit_count != 0:
                ordering.append(grad_fn)
                continue

            stack.append((grad_fn, 1))
            for next_fn, input_idx in grad_fn.next_functions:
                if next_fn is None:
                    continue

                if (exclude_accumulate_grad and
                        next_fn.name() == 'torch::autograd::AccumulateGrad'):
                    continue

                # Keep track of the inputs to each grad_fn
                if next_fn not in input_map:
                    input_map[next_fn] = 1
                input_map[next_fn] = max(input_map[next_fn], input_idx + 1)

                # Determine whether to visit this grad_fn
                if next_fn in visited:
                    continue

                visited.add(next_fn)
                stack.append((next_fn, 0))

        ordering.reverse()
        return cls(ordering, input_map, initial_inputs)

    def run_backward(self):
        for grad_fn in self._grad_fn_ordering:
            # 1. Run the backward function
            outputs = grad_fn(*(self._input_holder[grad_fn]))

            # 2. Store its outputs for the next backward function(s)
            if isinstance(outputs, torch.Tensor):
                outputs = [outputs]
            for (output, (next_fn, input_idx)) in zip(
                    outputs, grad_fn.next_functions):
                if next_fn is None or next_fn not in self._input_holder:
                    continue
                # NOTE: If implementing to actually calculate the gradient, we
                # need to sum gradients that "flow" into the same grad function
                # input.
                self._input_holder[next_fn][input_idx] = output



import unittest

import deepview_profile.lru_cache as lru


class LRUCacheTests(unittest.TestCase):
    def test_cache_empty(self):
        cache = lru.LRUCache()
        self.assertIsNone(cache.query(123))
        self.assertEqual(len(cache._cache_by_key), 0)
        self.assertEqual(cache._cache_by_use.size, 0)

    def test_cache_one(self):
        cache = lru.LRUCache(max_size=1)
        self.assertIsNone(cache.query(1))
        cache.add(1, 1)
        self.assertEqual(cache.query(1), 1)
        cache.add(2, 2)
        self.assertIsNone(cache.query(1))
        self.assertEqual(cache.query(2), 2)
        self.assertEqual(len(cache._cache_by_key), 1)
        self.assertEqual(cache._cache_by_use.size, 1)

    def test_cache_two(self):
        cache = lru.LRUCache(max_size=2)
        cache.add(1, 1)
        cache.add(2, 2)
        self.assertEqual(cache.query(2), 2)
        cache.add(3, 3)
        self.assertEqual(cache.query(2), 2)
        cache.add(5, 5)
        self.assertIsNone(cache.query(3))
        self.assertEqual(len(cache._cache_by_key), 2)
        self.assertEqual(cache._cache_by_use.size, 2)

    def test_cache_three(self):
        cache = lru.LRUCache(max_size=3)
        self.assertIsNone(cache.query(1))
        cache.add(1, 1)
        cache.add(2, 2)
        cache.add(3, 3)
        self.assertEqual(cache.query(1), 1)
        self.assertEqual(cache.query(3), 3)
        self.assertEqual(cache.query(2), 2)
        cache.add(4, 4)
        self.assertIsNone(cache.query(1))
        self.assertEqual(cache.query(4), 4)
        self.assertEqual(cache.query(3), 3)
        self.assertEqual(cache.query(2), 2)
        cache.add(5, 5)
        self.assertIsNone(cache.query(4))
        self.assertEqual(len(cache._cache_by_key), 3)
        self.assertEqual(cache._cache_by_use.size, 3)

    def test_cache_cyclical(self):
        cache = lru.LRUCache(max_size=3)
        cache.add(1, 1)
        cache.add(2, 2)
        cache.add(3, 3)
        cache.add(4, 4)
        cache.add(5, 5)
        cache.add(6, 6)
        self.assertIsNone(cache.query(1))
        self.assertIsNone(cache.query(2))
        self.assertIsNone(cache.query(3))
        self.assertEqual(cache.query(4), 4)
        self.assertEqual(cache.query(5), 5)
        self.assertEqual(cache.query(6), 6)
        self.assertEqual(len(cache._cache_by_key), 3)
        self.assertEqual(cache._cache_by_use.size, 3)


class LRUCacheListTests(unittest.TestCase):
    def setUp(self):
        self.list = lru._LRUCacheList()

    def list_to_array(self, backward=False):
        items = []
        ptr = self.list.front if not backward else self.list.back
        while ptr is not None:
            items.append((ptr.key, ptr.value))
            ptr = ptr.next if not backward else ptr.prev
        return items

    def test_list_none(self):
        self.assertIsNone(self.list.front)
        self.assertIsNone(self.list.back)
        self.assertEqual(self.list.size, 0)
        node = self.list.add_to_front(1, 1)
        self.assertIsNone(node.next)
        self.assertIsNone(node.prev)
        self.assertEqual(self.list.size, 1)

    def test_list_add(self):
        self.list.add_to_front('hello', 'world')
        self.assertEqual(self.list.size, 1)
        self.assertEqual(self.list.front, self.list.back)
        self.assertIsNone(self.list.front.next)
        self.assertIsNone(self.list.front.prev)
        self.assertEqual(self.list.front.key, 'hello')
        self.assertEqual(self.list.front.value, 'world')

        self.list.add_to_front('hello2', 'world2')
        self.assertEqual(self.list.size, 2)
        self.assertEqual(
            self.list_to_array(backward=True),
            [('hello', 'world'), ('hello2', 'world2')],
        )

    def test_list_add_several(self):
        self.list.add_to_front(1, 1)
        self.list.add_to_front(2, 2)
        self.list.add_to_front(3, 3)
        self.assertEqual(self.list.size, 3)
        self.assertNotEqual(self.list.front, self.list.back)
        self.assertEqual(self.list_to_array(), [(3, 3), (2, 2), (1, 1)])

    def test_list_move_three(self):
        n1 = self.list.add_to_front(1, 1)
        n2 = self.list.add_to_front(2, 2)
        self.list.add_to_front(3, 3)

        self.list.move_to_front(n1)
        self.assertEqual(self.list_to_array(), [(1, 1), (3, 3), (2, 2)])
        self.assertEqual(
            self.list_to_array(backward=True), [(2, 2), (3, 3), (1, 1)])
        self.assertEqual(self.list.size, 3)

        self.list.move_to_front(n2)
        self.assertEqual(self.list_to_array(), [(2, 2), (1, 1), (3, 3)])
        self.assertEqual(
            self.list_to_array(backward=True), [(3, 3), (1, 1), (2, 2)])
        self.assertEqual(self.list.size, 3)

        self.list.move_to_front(n2)
        self.assertEqual(self.list_to_array(), [(2, 2), (1, 1), (3, 3)])
        self.assertEqual(
            self.list_to_array(backward=True), [(3, 3), (1, 1), (2, 2)])
        self.assertEqual(self.list.size, 3)

    def test_list_move_two(self):
        n1 = self.list.add_to_front(1, 1)
        self.list.add_to_front(2, 2)
        self.assertEqual(self.list_to_array(), [(2, 2), (1, 1)])

        self.list.move_to_front(n1)
        self.assertEqual(self.list_to_array(), [(1, 1), (2, 2)])

    def test_list_move_one(self):
        n1 = self.list.add_to_front(1, 1)
        self.assertEqual(self.list_to_array(), [(1, 1)])
        self.list.move_to_front(n1)
        self.assertEqual(self.list_to_array(), [(1, 1)])
        self.assertEqual(self.list_to_array(backward=True), [(1, 1)])
        self.assertEqual(self.list.size, 1)

    def test_list_remove_back_empty(self):
        removed = self.list.remove_back()
        self.assertEqual(self.list.size, 0)
        self.assertIsNone(removed)
        self.assertEqual(self.list_to_array(), [])

    def test_list_remove_back_one(self):
        n1 = self.list.add_to_front(1, 1)
        self.assertEqual(self.list_to_array(), [(1, 1)])
        removed = self.list.remove_back()
        self.assertEqual(n1, removed)
        self.assertEqual(self.list.size, 0)
        self.assertEqual(self.list_to_array(), [])

    def test_list_remove_back_two(self):
        n1 = self.list.add_to_front(1, 1)
        self.list.add_to_front(2, 2)
        self.assertEqual(self.list.size, 2)
        self.assertEqual(self.list_to_array(), [(2, 2), (1, 1)])
        removed = self.list.remove_back()
        self.assertEqual(removed, n1)
        self.assertEqual(self.list_to_array(), [(2, 2)])
        self.assertEqual(self.list_to_array(backward=True), [(2, 2)])
        self.assertEqual(self.list.size, 1)

    def test_list_remove_back_three(self):
        n1 = self.list.add_to_front(1, 1)
        self.list.add_to_front(2, 2)
        self.list.add_to_front(3, 3)
        self.assertEqual(self.list.size, 3)
        self.assertEqual(self.list_to_array(), [(3, 3), (2, 2), (1, 1)])
        removed = self.list.remove_back()
        self.assertEqual(removed, n1)
        self.assertEqual(self.list_to_array(), [(3, 3), (2, 2)])
        self.assertEqual(self.list_to_array(backward=True), [(2, 2), (3, 3)])
        self.assertEqual(self.list.size, 2)



import torch
import inspect

from deepview_profile.tracking.base import TrackerBase
from deepview_profile.tracking.call_stack import CallStack
from deepview_profile.tracking.hook_manager import HookManager
from deepview_profile.tracking.utils import tensor_size_bytes
from deepview_profile.util_weak import WeakTensorKeyDictionary

class WeightsTracker(TrackerBase):
    def __init__(self, project_root):
        super().__init__()
        self._hook_manager = HookManager()
        self._module_parameters = WeakTensorKeyDictionary()
        self._project_root = project_root

    def start_tracking(self):
        super().start_tracking()
        self._hook_manager.attach_hook(
            torch.nn.Module,
            'register_parameter',
            self._register_parameter_hook_creator,
        )

    def stop_tracking(self):
        super().stop_tracking()
        self._hook_manager.remove_hooks()

    def populate_report(self, builder):
        for param, (name, stack) in self._module_parameters.items():
            if not param.is_cuda:
                continue
            builder.add_weight_entry(
                weight_name=name,
                size_bytes=tensor_size_bytes(param),
                grad_size_bytes=tensor_size_bytes(param.grad),
                stack_context=stack,
            )

    def populate_breakdown(self, builder):
        # The HierarchicalBreakdownBuilder uses the same API as the
        # MemoryReportBuilder.
        self.populate_report(builder)

    def _register_parameter_hook_creator(self, func):
        def hook(*args, **kwargs):
            name = args[1]
            parameter = args[2]
            retval = func(*args, **kwargs)
            if parameter is not None and parameter not in self._module_parameters:
                self._module_parameters[parameter] = (
                    name,
                    CallStack.from_here(self._project_root, start_from=2),
                )
            return retval
        return hook



import contextlib
import sys

from deepview_profile.exceptions import exceptions_as_analysis_errors


@contextlib.contextmanager
def user_code_environment(script_root_path, project_root):
    """
    A combined context manager that activates all relevant context managers
    used when running user code.
    """
    with sys_path_root(script_root_path):
        # with prevent_module_caching():
            with exceptions_as_analysis_errors(project_root):
                yield


@contextlib.contextmanager
def sys_path_root(script_root_path):
    """
    A context manager that sets sys.path[0] to the specified path on entry and
    then restores it after exiting the context manager.
    """
    # As per the Python documentation, sys.path[0] always stores the path to
    # the directory containing the Python script that was used to start the
    # Python interpreter. The contents of sys.path are used to resolve module
    # imports.
    #
    # When we run user code (e.g., the user's entry point file), we want to run
    # it as if it was being directly executed by the user from the shell. For
    # example:
    #
    #   $ python3 entry_point.py
    #
    # For this to work, we need to ensure that sys.path[0] is the path to the
    # directory containing the entry_point.py file. However if we use exec(),
    # sys.path[0] is set to the path of DeepView's command line executable.
    #
    # To fix this problem, we set sys.path[0] to the correct root path before
    # running the user's code and restore it to DeepView's script path after the
    # execution completes. Doing this is **very important** as it ensures that
    # imports work as expected inside the user's code. This context manager
    # should be used each time we execute user code because imports can exist
    # inside user-defined functions.
    #
    # Setting and then restoring sys.path[0] is better than just appending the
    # user's path to sys.path because we want to avoid accidentally importing
    # anything from the user's codebase.
    skyline_script_root = sys.path[0]
    try:
        sys.path[0] = script_root_path
        yield
    finally:
        sys.path[0] = skyline_script_root


@contextlib.contextmanager
def prevent_module_caching():
    """
    A context manager that prevents any imported modules from being cached
    after exiting.
    """
    try:
        original_modules = sys.modules.copy()
        yield
    finally:
        newly_added = {
            module_name for module_name in sys.modules.keys()
            if module_name not in original_modules
        }
        for module_name in newly_added:
            del sys.modules[module_name]



import collections
import inspect
import os
import re
import torch
from deepview_profile.utils import model_location_patterns

SourceLocation = collections.namedtuple(
    "SourceLocation", ["file_path", "line_number", "module_id"]
)

def find_pattern_match(filename):
    pattern_list = model_location_patterns()
    return any(re.search(pattern, filename) for pattern in pattern_list)

class CallStack:
    def __init__(self, frames):
        self.frames = frames

    @staticmethod
    def from_here(project_root, start_from=1):
        """
        Returns the current call stack when invoked.
        """
        stack = inspect.stack()
        context = []
        try:
            for frame_info in stack[start_from:]:
                # Only track source locations that are within the project model (or transformers, diffusers, etc)
                # that are within a torch.nn.Module. Note that we assume the
                # user uses "self" to refer to the current class instance.

                if not (
                    frame_info.filename.startswith(project_root)
                    or find_pattern_match(frame_info.filename)
                ):
                    continue
                if "self" not in frame_info.frame.f_locals:
                    continue
                if not isinstance(frame_info.frame.f_locals["self"], torch.nn.Module):
                    continue
                context.append(
                    SourceLocation(
                        file_path=os.path.relpath(
                            frame_info.filename, start=project_root
                        ),
                        line_number=frame_info.lineno,
                        module_id=id(frame_info.frame.f_locals["self"]),
                    )
                )
            return CallStack(context)
        finally:
            del stack



import ast
import re

END_OF_FUNCTION = re.compile('\):\s*$')


class StaticAnalyzer:
    def __init__(self, source_code, source_tree):
        self._ast = source_tree
        self._code_by_line = source_code.splitlines()

    def batch_size_location(self):
        """
        Locates the line of the 'batch_size' argument in the
        'skyline_input_provider' function and determines if the provider's
        definition can be mutated using our heuristics.
        """
        extractor = _InputProviderExtractor()
        extractor.visit(self._ast)
        function = extractor.function_node

        if (function is None or
                len(function.args.args) == 0 or
                function.args.args[0].arg != 'batch_size'):
            return None

        batch_size_line_number = function.args.args[0].lineno
        match = END_OF_FUNCTION.search(self._code_by_line[function.lineno - 1])
        can_mutate = match is not None

        return batch_size_line_number, can_mutate


class _InputProviderExtractor(ast.NodeVisitor):
    def __init__(self):
        self.function_node = None

    def visit_FunctionDef(self, node):
        if self.function_node is not None:
            # Return early if we've already found the provider
            return
        if node.name != 'deepview_input_provider':
            return
        self.function_node = node





create_report_tables = {
    'run_time_entries': """
      CREATE TABLE IF NOT EXISTS run_time_entries (
        id INTEGER PRIMARY KEY,
        operation_name TEXT NOT NULL,
        forward_ms REAL NOT NULL,
        backward_ms REAL
      )
    """,
    'stack_frames': """
      CREATE TABLE IF NOT EXISTS stack_frames (
        ordering INTEGER NOT NULL,
        file_path TEXT NOT NULL,
        line_number INTEGER NOT NULL,
        entry_id INTEGER NOT NULL,
        PRIMARY KEY (entry_id, ordering)
      )
    """,
}

set_report_format_version = 'PRAGMA user_version = {version:d}'

add_stack_frame = """
  INSERT INTO stack_frames (ordering, file_path, line_number, entry_id)
    VALUES (?, ?, ?, ?)
"""

add_run_time_entry = """
  INSERT INTO run_time_entries (operation_name, forward_ms, backward_ms)
    VALUES (?, ?, ?)
"""

get_run_time_entries_with_context = """
  WITH code_contexts AS (
    SELECT entry_id, file_path, line_number FROM stack_frames
    GROUP BY entry_id HAVING ordering == MIN(ordering)
  )
  SELECT
    e.operation_name,
    e.forward_ms,
    e.backward_ms,
    c.file_path,
    c.line_number
  FROM
    run_time_entries AS e LEFT JOIN code_contexts AS c
    ON e.id == c.entry_id
  ORDER BY c.file_path ASC, c.line_number ASC
"""



import argparse
import sys


import deepview_profile
import deepview_profile.commands.interactive
import deepview_profile.commands.memory
import deepview_profile.commands.time


def main():
    parser = argparse.ArgumentParser(
        prog = deepview_profile.__name__,
        description = deepview_profile.__description__
    )
    parser.add_argument(
        "-v", "--version",
        action="store_true",
        help="Print the version and exit.",
    )
    subparsers = parser.add_subparsers(title="Commands")
    deepview_profile.commands.interactive.register_command(subparsers)
    deepview_profile.commands.memory.register_command(subparsers)
    deepview_profile.commands.time.register_command(subparsers)
    args = parser.parse_args()

    if args.version:
        print('DeepView Command Line Interface', 'v' + deepview_profile.__version__,)
        return

    if 'func' not in args:
        parser.print_help()
        sys.exit(1)

    # Run the specified command
    args.func(args)


if __name__ == '__main__':
    main()



import logging
import os
import sys

from deepview_profile.initialization import (
    check_skyline_preconditions,
    initialize_skyline,
)
from deepview_profile.error_printing import print_analysis_error

logger = logging.getLogger(__name__)


def register_command(subparsers):
    parser = subparsers.add_parser(
        "memory",
        help="Generate a memory usage report.",
    )
    parser.add_argument(
        "entry_point",
        help="The entry point file in this project that contains the DeepView "
             "provider functions.",
    )
    parser.add_argument(
        "-o", "--output",
        help="The location where the memory report should be stored.",
        required=True,
    )
    parser.add_argument(
        "--log-file",
        help="The location of the log file.",
    )
    parser.add_argument(
        "--debug", action="store_true", help="Log debug messages.")
    parser.set_defaults(func=main)


def actual_main(args):
    from deepview_profile.analysis.session import AnalysisSession
    from deepview_profile.exceptions import AnalysisError

    if os.path.exists(args.output):
        print(
            "ERROR: The specified output file already exists.",
            file=sys.stderr,
        )
        sys.exit(1)

    try:
        project_root = os.getcwd()
        session = AnalysisSession.new_from(
            project_root, args.entry_point)
        session.generate_memory_usage_report(
            save_report_to=args.output,
        )
    except AnalysisError as ex:
        print_analysis_error(ex)
        sys.exit(1)


def main(args):
    check_skyline_preconditions(args)
    initialize_skyline(args)
    actual_main(args)



import logging
import select
import struct
from threading import Thread

from deepview_profile.io.sentinel import Sentinel

logger = logging.getLogger(__name__)


class Connection:
    """
    Manages an open connection to a client.

    This class must be constructed with an already-connected
    socket. Upon receipt of a message on the socket, the
    handler_function will be called with the raw message.

    Socket communication is performed using length-prefixed
    binary protobuf messages.

    The stop function must be called to close the connection.
    """
    def __init__(self, socket, address, handler_function, closed_handler):
        self.address = address
        self._socket = socket
        self._reader = Thread(target=self._socket_read)
        self._handler_function = handler_function
        self._closed_handler = closed_handler
        self._sentinel = Sentinel()
        self._project_root = ""
        self._entry_point = ""

    def start(self):
        self._sentinel.start()
        self._reader.start()

    def stop(self):
        self._sentinel.signal_exit()
        self._reader.join()
        self._socket.close()
        self._sentinel.stop()

    def send_bytes(self, raw_bytes):
        self._socket.sendall(struct.pack('!I', len(raw_bytes)))
        self._socket.sendall(raw_bytes)

    def _socket_read(self):
        buffer = b''
        message_length = -1

        try:
            while True:
                read_ready, _, _ = select.select([
                    self._socket, self._sentinel.read_pipe], [], [])
                if self._sentinel.should_exit(read_ready):
                    logger.debug(
                        "Connection (%s:%d) is being closed.",
                        *self.address,
                    )
                    self._sentinel.consume_exit_signal()
                    break

                data = self._socket.recv(4096)
                if len(data) == 0:
                    logger.debug(
                        "Connection (%s:%d) has been closed by the client.",
                        *self.address,
                    )
                    self._closed_handler(self.address)
                    break

                buffer += data

                # Process all messages that exist in the buffer
                while True:
                    if message_length <= 0:
                        if len(buffer) < 4:
                            break
                        # Network byte order 32-bit unsigned integer
                        message_length = struct.unpack('!I', buffer[:4])[0]
                        buffer = buffer[4:]

                    if len(buffer) < message_length:
                        break

                    try:
                        self._handler_function(
                            buffer[:message_length], self.address)
                    finally:
                        buffer = buffer[message_length:]
                        message_length = -1

        except Exception:
            logger.exception("Connection unexpectedly stopping...")

    @property
    def project_root(self):
        return self._project_root

    @property
    def entry_point(self):
        return self._entry_point

    def set_project_paths(self, project_root, entry_point):
        self._project_root = project_root
        self._entry_point = entry_point

class ConnectionState:
    def __init__(self):
        # NOTE: This counter is modified by a thread in the main executor, but
        #       will be read by other threads. No R/W lock is needed because of
        #       the Python GIL.
        #
        # NOTE: The sequence number from the client must be non-negative
        self.sequence_number = -1

        # Connections have two states: uninitialized and "ready" (initialized)
        # As a result for simplicity, we use a boolean to represent the state.
        self.initialized = False

        # The plugin may disconnect from us while we are processing a request.
        # We use this flag to indicate whether the connection still "exists"
        # to allow requests to abort early.
        self.connected = True

    def update_sequence(self, request):
        if request.sequence_number <= self.sequence_number:
            return
        self.sequence_number = request.sequence_number

    def is_request_current(self, request):
        return request.sequence_number >= self.sequence_number



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: innpv.proto

from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='innpv.proto',
  package='innpv.protocol',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0binnpv.proto\x12\x0einnpv.protocol\"\xcf\x01\n\nFromClient\x12\x17\n\x0fsequence_number\x18\x01 \x01(\r\x12\x37\n\ninitialize\x18\x02 \x01(\x0b\x32!.innpv.protocol.InitializeRequestH\x00\x12\x33\n\x08\x61nalysis\x18\x03 \x01(\x0b\x32\x1f.innpv.protocol.AnalysisRequestH\x00\x12/\n\x07generic\x18\x04 \x01(\x0b\x32\x1c.innpv.protocol.GenericEventH\x00\x42\t\n\x07payload\">\n\x0cGenericEvent\x12\x12\n\nevent_type\x18\x01 \x01(\t\x12\x1a\n\x12optional_arguments\x18\x02 \x01(\t\"X\n\x11InitializeRequest\x12\x18\n\x10protocol_version\x18\x01 \x01(\r\x12\x14\n\x0cproject_root\x18\x02 \x01(\t\x12\x13\n\x0b\x65ntry_point\x18\x03 \x01(\t\"F\n\x0f\x41nalysisRequest\x12\x15\n\rmock_response\x18\x01 \x01(\x08\x12\x1c\n\x14\x64\x64p_analysis_request\x18\x02 \x01(\x08\"\xca\x04\n\nFromServer\x12\x17\n\x0fsequence_number\x18\x01 \x01(\r\x12.\n\x05\x65rror\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.ProtocolErrorH\x00\x12\x38\n\ninitialize\x18\x03 \x01(\x0b\x32\".innpv.protocol.InitializeResponseH\x00\x12\x37\n\x0e\x61nalysis_error\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisErrorH\x00\x12\x38\n\nthroughput\x18\x06 \x01(\x0b\x32\".innpv.protocol.ThroughputResponseH\x00\x12\x36\n\tbreakdown\x18\x08 \x01(\x0b\x32!.innpv.protocol.BreakdownResponseH\x00\x12\x32\n\x07habitat\x18\t \x01(\x0b\x32\x1f.innpv.protocol.HabitatResponseH\x00\x12\x30\n\x06\x65nergy\x18\n \x01(\x0b\x32\x1e.innpv.protocol.EnergyResponseH\x00\x12:\n\x0butilization\x18\x0b \x01(\x0b\x32#.innpv.protocol.UtilizationResponseH\x00\x12=\n\x03\x64\x64p\x18\x0c \x01(\x0b\x32..innpv.protocol.DDPBucketSizesComputationTimesH\x00\x42\t\n\x07payloadJ\x04\x08\x04\x10\x05J\x04\x08\x07\x10\x08R\x0cmemory_usageR\x08run_time\"B\n\x17HabitatDevicePrediction\x12\x13\n\x0b\x64\x65vice_name\x18\x01 \x01(\t\x12\x12\n\nruntime_ms\x18\x02 \x01(\x02\"\x86\x01\n\x0fHabitatResponse\x12<\n\x0bpredictions\x18\x01 \x03(\x0b\x32\'.innpv.protocol.HabitatDevicePrediction\x12\x35\n\x0e\x61nalysis_error\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"\xf1\x01\n\x0e\x45nergyResponse\x12\x19\n\x11total_consumption\x18\x01 \x01(\x02\x12>\n\ncomponents\x18\x02 \x03(\x0b\x32*.innpv.protocol.EnergyConsumptionComponent\x12\x12\n\nbatch_size\x18\x03 \x01(\x05\x12\x39\n\x11past_measurements\x18\x04 \x03(\x0b\x32\x1e.innpv.protocol.EnergyResponse\x12\x35\n\x0e\x61nalysis_error\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"\x80\x01\n\x1a\x45nergyConsumptionComponent\x12\x46\n\x0e\x63omponent_type\x18\x01 \x01(\x0e\x32..innpv.protocol.EnergyConsumptionComponentType\x12\x1a\n\x12\x63onsumption_joules\x18\x02 \x01(\x02\"\x9b\x01\n\x13UtilizationResponse\x12\x31\n\x08rootNode\x18\x01 \x01(\x0b\x32\x1f.innpv.protocol.UtilizationNode\x12\x35\n\x0e\x61nalysis_error\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\x12\x1a\n\x12tensor_utilization\x18\x03 \x01(\x02\"\xd5\x01\n\x1e\x44\x44PBucketSizesComputationTimes\x12\x17\n\x0f\x66orward_time_ms\x18\x01 \x01(\x02\x12\x14\n\x0c\x62ucket_sizes\x18\x02 \x03(\x02\x12M\n\x11\x63omputation_times\x18\x03 \x03(\x0b\x32\x32.innpv.protocol.DDPExpectedMaximumComputationTimes\x12\x35\n\x0e\x61nalysis_error\x18\x04 \x01(\x0b\x32\x1d.innpv.protocol.AnalysisError\"O\n\"DDPExpectedMaximumComputationTimes\x12\r\n\x05ngpus\x18\x01 \x01(\x05\x12\x1a\n\x12\x65xpected_max_times\x18\x02 \x03(\x02\"\x8c\x01\n\x12InitializeResponse\x12\x1b\n\x13server_project_root\x18\x01 \x01(\t\x12)\n\x0b\x65ntry_point\x18\x02 \x01(\x0b\x32\x14.innpv.protocol.Path\x12.\n\x08hardware\x18\x03 \x01(\x0b\x32\x1c.innpv.protocol.HardwareInfo\"[\n\rAnalysisError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12\x33\n\x0c\x66ile_context\x18\x02 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\"\xa1\x02\n\x12ThroughputResponse\x12\x1a\n\x12samples_per_second\x18\x01 \x01(\x02\x12(\n predicted_max_samples_per_second\x18\x02 \x01(\x02\x12\x30\n\x0brun_time_ms\x18\x03 \x01(\x0b\x32\x1b.innpv.protocol.LinearModel\x12\x35\n\x10peak_usage_bytes\x18\x04 \x01(\x0b\x32\x1b.innpv.protocol.LinearModel\x12\x39\n\x12\x62\x61tch_size_context\x18\x05 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\x12!\n\x19\x63\x61n_manipulate_batch_size\x18\x06 \x01(\x08\"\xea\x01\n\x11\x42reakdownResponse\x12\x18\n\x10peak_usage_bytes\x18\x01 \x01(\x04\x12\x1d\n\x15memory_capacity_bytes\x18\x02 \x01(\x04\x12\x1d\n\x15iteration_run_time_ms\x18\x03 \x01(\x02\x12\x12\n\nbatch_size\x18\x06 \x01(\r\x12\x35\n\x0eoperation_tree\x18\x04 \x03(\x0b\x32\x1d.innpv.protocol.BreakdownNode\x12\x32\n\x0bweight_tree\x18\x05 \x03(\x0b\x32\x1d.innpv.protocol.BreakdownNode\"\xca\x01\n\rProtocolError\x12;\n\nerror_code\x18\x01 \x01(\x0e\x32\'.innpv.protocol.ProtocolError.ErrorCode\"|\n\tErrorCode\x12\x0b\n\x07UNKNOWN\x10\x00\x12 \n\x1cUNSUPPORTED_PROTOCOL_VERSION\x10\x01\x12\x1c\n\x18UNINITIALIZED_CONNECTION\x10\x02\x12\"\n\x1e\x41LREADY_INITIALIZED_CONNECTION\x10\x03\"\x1a\n\x04Path\x12\x12\n\ncomponents\x18\x01 \x03(\t\"M\n\rFileReference\x12\'\n\tfile_path\x18\x01 \x01(\x0b\x32\x14.innpv.protocol.Path\x12\x13\n\x0bline_number\x18\x02 \x01(\r\"\xce\x01\n\rBreakdownNode\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0cnum_children\x18\x02 \x01(\r\x12/\n\x08\x63ontexts\x18\x03 \x03(\x0b\x32\x1d.innpv.protocol.FileReference\x12\x32\n\toperation\x18\x04 \x01(\x0b\x32\x1d.innpv.protocol.OperationDataH\x00\x12,\n\x06weight\x18\x05 \x01(\x0b\x32\x1a.innpv.protocol.WeightDataH\x00\x42\x06\n\x04\x64\x61ta\"{\n\x0b\x43ontextInfo\x12.\n\x07\x63ontext\x18\x01 \x01(\x0b\x32\x1d.innpv.protocol.FileReference\x12\x13\n\x0brun_time_ms\x18\x02 \x01(\x02\x12\x12\n\nsize_bytes\x18\x03 \x01(\x04\x12\x13\n\x0binvocations\x18\x04 \x01(\r\"\x83\x01\n\rOperationData\x12\x12\n\nforward_ms\x18\x01 \x01(\x02\x12\x13\n\x0b\x62\x61\x63kward_ms\x18\x02 \x01(\x02\x12\x12\n\nsize_bytes\x18\x03 \x01(\x04\x12\x35\n\x10\x63ontext_info_map\x18\x04 \x03(\x0b\x32\x1b.innpv.protocol.ContextInfo\"9\n\nWeightData\x12\x12\n\nsize_bytes\x18\x01 \x01(\x04\x12\x17\n\x0fgrad_size_bytes\x18\x02 \x01(\x04\"*\n\x0bLinearModel\x12\r\n\x05slope\x18\x01 \x01(\x01\x12\x0c\n\x04\x62ias\x18\x02 \x01(\x01\":\n\x0cHardwareInfo\x12\x10\n\x08hostname\x18\x01 \x01(\t\x12\n\n\x02os\x18\x02 \x01(\t\x12\x0c\n\x04gpus\x18\x03 \x03(\t\"\xc0\x02\n\x0fUtilizationNode\x12\x10\n\x08slice_id\x18\x01 \x01(\r\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05start\x18\x03 \x01(\x04\x12\x0b\n\x03\x65nd\x18\x04 \x01(\x04\x12\x13\n\x0b\x63pu_forward\x18\x05 \x01(\x04\x12\x18\n\x10\x63pu_forward_span\x18\x06 \x01(\x04\x12\x13\n\x0bgpu_forward\x18\x07 \x01(\x04\x12\x18\n\x10gpu_forward_span\x18\x08 \x01(\x04\x12\x14\n\x0c\x63pu_backward\x18\t \x01(\x04\x12\x19\n\x11\x63pu_backward_span\x18\n \x01(\x04\x12\x14\n\x0cgpu_backward\x18\x0b \x01(\x04\x12\x19\n\x11gpu_backward_span\x18\x0c \x01(\x04\x12\x31\n\x08\x63hildren\x18\r \x03(\x0b\x32\x1f.innpv.protocol.UtilizationNode\"\x1b\n\x13MemoryUsageResponseJ\x04\x08\x01\x10\x65\"\x17\n\x0fRunTimeResponseJ\x04\x08\x01\x10\x65\"\x17\n\x0f\x41\x63tivationEntryJ\x04\x08\x01\x10\x65\"\x13\n\x0bWeightEntryJ\x04\x08\x01\x10\x65\"\x14\n\x0cRunTimeEntryJ\x04\x08\x01\x10\x65*`\n\x1e\x45nergyConsumptionComponentType\x12\x16\n\x12\x45NERGY_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x45NERGY_CPU_DRAM\x10\x01\x12\x11\n\rENERGY_NVIDIA\x10\x02\x62\x06proto3'
)

_ENERGYCONSUMPTIONCOMPONENTTYPE = _descriptor.EnumDescriptor(
  name='EnergyConsumptionComponentType',
  full_name='innpv.protocol.EnergyConsumptionComponentType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ENERGY_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENERGY_CPU_DRAM', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ENERGY_NVIDIA', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=4244,
  serialized_end=4340,
)
_sym_db.RegisterEnumDescriptor(_ENERGYCONSUMPTIONCOMPONENTTYPE)

EnergyConsumptionComponentType = enum_type_wrapper.EnumTypeWrapper(_ENERGYCONSUMPTIONCOMPONENTTYPE)
ENERGY_UNSPECIFIED = 0
ENERGY_CPU_DRAM = 1
ENERGY_NVIDIA = 2


_PROTOCOLERROR_ERRORCODE = _descriptor.EnumDescriptor(
  name='ErrorCode',
  full_name='innpv.protocol.ProtocolError.ErrorCode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNKNOWN', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNSUPPORTED_PROTOCOL_VERSION', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='UNINITIALIZED_CONNECTION', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALREADY_INITIALIZED_CONNECTION', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2935,
  serialized_end=3059,
)
_sym_db.RegisterEnumDescriptor(_PROTOCOLERROR_ERRORCODE)


_FROMCLIENT = _descriptor.Descriptor(
  name='FromClient',
  full_name='innpv.protocol.FromClient',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence_number', full_name='innpv.protocol.FromClient.sequence_number', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initialize', full_name='innpv.protocol.FromClient.initialize', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis', full_name='innpv.protocol.FromClient.analysis', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='generic', full_name='innpv.protocol.FromClient.generic', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='payload', full_name='innpv.protocol.FromClient.payload',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=32,
  serialized_end=239,
)


_GENERICEVENT = _descriptor.Descriptor(
  name='GenericEvent',
  full_name='innpv.protocol.GenericEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='event_type', full_name='innpv.protocol.GenericEvent.event_type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='optional_arguments', full_name='innpv.protocol.GenericEvent.optional_arguments', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=241,
  serialized_end=303,
)


_INITIALIZEREQUEST = _descriptor.Descriptor(
  name='InitializeRequest',
  full_name='innpv.protocol.InitializeRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='protocol_version', full_name='innpv.protocol.InitializeRequest.protocol_version', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='project_root', full_name='innpv.protocol.InitializeRequest.project_root', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entry_point', full_name='innpv.protocol.InitializeRequest.entry_point', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=305,
  serialized_end=393,
)


_ANALYSISREQUEST = _descriptor.Descriptor(
  name='AnalysisRequest',
  full_name='innpv.protocol.AnalysisRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mock_response', full_name='innpv.protocol.AnalysisRequest.mock_response', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ddp_analysis_request', full_name='innpv.protocol.AnalysisRequest.ddp_analysis_request', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=395,
  serialized_end=465,
)


_FROMSERVER = _descriptor.Descriptor(
  name='FromServer',
  full_name='innpv.protocol.FromServer',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sequence_number', full_name='innpv.protocol.FromServer.sequence_number', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='innpv.protocol.FromServer.error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='initialize', full_name='innpv.protocol.FromServer.initialize', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.FromServer.analysis_error', index=3,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='throughput', full_name='innpv.protocol.FromServer.throughput', index=4,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='breakdown', full_name='innpv.protocol.FromServer.breakdown', index=5,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='habitat', full_name='innpv.protocol.FromServer.habitat', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='energy', full_name='innpv.protocol.FromServer.energy', index=7,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='utilization', full_name='innpv.protocol.FromServer.utilization', index=8,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ddp', full_name='innpv.protocol.FromServer.ddp', index=9,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='payload', full_name='innpv.protocol.FromServer.payload',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=468,
  serialized_end=1054,
)


_HABITATDEVICEPREDICTION = _descriptor.Descriptor(
  name='HabitatDevicePrediction',
  full_name='innpv.protocol.HabitatDevicePrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='device_name', full_name='innpv.protocol.HabitatDevicePrediction.device_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='runtime_ms', full_name='innpv.protocol.HabitatDevicePrediction.runtime_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1056,
  serialized_end=1122,
)


_HABITATRESPONSE = _descriptor.Descriptor(
  name='HabitatResponse',
  full_name='innpv.protocol.HabitatResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='predictions', full_name='innpv.protocol.HabitatResponse.predictions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.HabitatResponse.analysis_error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1125,
  serialized_end=1259,
)


_ENERGYRESPONSE = _descriptor.Descriptor(
  name='EnergyResponse',
  full_name='innpv.protocol.EnergyResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='total_consumption', full_name='innpv.protocol.EnergyResponse.total_consumption', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='components', full_name='innpv.protocol.EnergyResponse.components', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='innpv.protocol.EnergyResponse.batch_size', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='past_measurements', full_name='innpv.protocol.EnergyResponse.past_measurements', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.EnergyResponse.analysis_error', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1262,
  serialized_end=1503,
)


_ENERGYCONSUMPTIONCOMPONENT = _descriptor.Descriptor(
  name='EnergyConsumptionComponent',
  full_name='innpv.protocol.EnergyConsumptionComponent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='component_type', full_name='innpv.protocol.EnergyConsumptionComponent.component_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='consumption_joules', full_name='innpv.protocol.EnergyConsumptionComponent.consumption_joules', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1506,
  serialized_end=1634,
)


_UTILIZATIONRESPONSE = _descriptor.Descriptor(
  name='UtilizationResponse',
  full_name='innpv.protocol.UtilizationResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='rootNode', full_name='innpv.protocol.UtilizationResponse.rootNode', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.UtilizationResponse.analysis_error', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tensor_utilization', full_name='innpv.protocol.UtilizationResponse.tensor_utilization', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1637,
  serialized_end=1792,
)


_DDPBUCKETSIZESCOMPUTATIONTIMES = _descriptor.Descriptor(
  name='DDPBucketSizesComputationTimes',
  full_name='innpv.protocol.DDPBucketSizesComputationTimes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='forward_time_ms', full_name='innpv.protocol.DDPBucketSizesComputationTimes.forward_time_ms', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bucket_sizes', full_name='innpv.protocol.DDPBucketSizesComputationTimes.bucket_sizes', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='computation_times', full_name='innpv.protocol.DDPBucketSizesComputationTimes.computation_times', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='analysis_error', full_name='innpv.protocol.DDPBucketSizesComputationTimes.analysis_error', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1795,
  serialized_end=2008,
)


_DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES = _descriptor.Descriptor(
  name='DDPExpectedMaximumComputationTimes',
  full_name='innpv.protocol.DDPExpectedMaximumComputationTimes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ngpus', full_name='innpv.protocol.DDPExpectedMaximumComputationTimes.ngpus', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='expected_max_times', full_name='innpv.protocol.DDPExpectedMaximumComputationTimes.expected_max_times', index=1,
      number=2, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2010,
  serialized_end=2089,
)


_INITIALIZERESPONSE = _descriptor.Descriptor(
  name='InitializeResponse',
  full_name='innpv.protocol.InitializeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='server_project_root', full_name='innpv.protocol.InitializeResponse.server_project_root', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entry_point', full_name='innpv.protocol.InitializeResponse.entry_point', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='hardware', full_name='innpv.protocol.InitializeResponse.hardware', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2092,
  serialized_end=2232,
)


_ANALYSISERROR = _descriptor.Descriptor(
  name='AnalysisError',
  full_name='innpv.protocol.AnalysisError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_message', full_name='innpv.protocol.AnalysisError.error_message', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='file_context', full_name='innpv.protocol.AnalysisError.file_context', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2234,
  serialized_end=2325,
)


_THROUGHPUTRESPONSE = _descriptor.Descriptor(
  name='ThroughputResponse',
  full_name='innpv.protocol.ThroughputResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='samples_per_second', full_name='innpv.protocol.ThroughputResponse.samples_per_second', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='predicted_max_samples_per_second', full_name='innpv.protocol.ThroughputResponse.predicted_max_samples_per_second', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='run_time_ms', full_name='innpv.protocol.ThroughputResponse.run_time_ms', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='peak_usage_bytes', full_name='innpv.protocol.ThroughputResponse.peak_usage_bytes', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size_context', full_name='innpv.protocol.ThroughputResponse.batch_size_context', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='can_manipulate_batch_size', full_name='innpv.protocol.ThroughputResponse.can_manipulate_batch_size', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2328,
  serialized_end=2617,
)


_BREAKDOWNRESPONSE = _descriptor.Descriptor(
  name='BreakdownResponse',
  full_name='innpv.protocol.BreakdownResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='peak_usage_bytes', full_name='innpv.protocol.BreakdownResponse.peak_usage_bytes', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='memory_capacity_bytes', full_name='innpv.protocol.BreakdownResponse.memory_capacity_bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iteration_run_time_ms', full_name='innpv.protocol.BreakdownResponse.iteration_run_time_ms', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='innpv.protocol.BreakdownResponse.batch_size', index=3,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation_tree', full_name='innpv.protocol.BreakdownResponse.operation_tree', index=4,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weight_tree', full_name='innpv.protocol.BreakdownResponse.weight_tree', index=5,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2620,
  serialized_end=2854,
)


_PROTOCOLERROR = _descriptor.Descriptor(
  name='ProtocolError',
  full_name='innpv.protocol.ProtocolError',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='error_code', full_name='innpv.protocol.ProtocolError.error_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PROTOCOLERROR_ERRORCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2857,
  serialized_end=3059,
)


_PATH = _descriptor.Descriptor(
  name='Path',
  full_name='innpv.protocol.Path',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='components', full_name='innpv.protocol.Path.components', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3061,
  serialized_end=3087,
)


_FILEREFERENCE = _descriptor.Descriptor(
  name='FileReference',
  full_name='innpv.protocol.FileReference',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='file_path', full_name='innpv.protocol.FileReference.file_path', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='line_number', full_name='innpv.protocol.FileReference.line_number', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3089,
  serialized_end=3166,
)


_BREAKDOWNNODE = _descriptor.Descriptor(
  name='BreakdownNode',
  full_name='innpv.protocol.BreakdownNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='innpv.protocol.BreakdownNode.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='num_children', full_name='innpv.protocol.BreakdownNode.num_children', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='contexts', full_name='innpv.protocol.BreakdownNode.contexts', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='operation', full_name='innpv.protocol.BreakdownNode.operation', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='weight', full_name='innpv.protocol.BreakdownNode.weight', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='data', full_name='innpv.protocol.BreakdownNode.data',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=3169,
  serialized_end=3375,
)


_CONTEXTINFO = _descriptor.Descriptor(
  name='ContextInfo',
  full_name='innpv.protocol.ContextInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='context', full_name='innpv.protocol.ContextInfo.context', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='run_time_ms', full_name='innpv.protocol.ContextInfo.run_time_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.ContextInfo.size_bytes', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='invocations', full_name='innpv.protocol.ContextInfo.invocations', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3377,
  serialized_end=3500,
)


_OPERATIONDATA = _descriptor.Descriptor(
  name='OperationData',
  full_name='innpv.protocol.OperationData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='forward_ms', full_name='innpv.protocol.OperationData.forward_ms', index=0,
      number=1, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='backward_ms', full_name='innpv.protocol.OperationData.backward_ms', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.OperationData.size_bytes', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='context_info_map', full_name='innpv.protocol.OperationData.context_info_map', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3503,
  serialized_end=3634,
)


_WEIGHTDATA = _descriptor.Descriptor(
  name='WeightData',
  full_name='innpv.protocol.WeightData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='innpv.protocol.WeightData.size_bytes', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='grad_size_bytes', full_name='innpv.protocol.WeightData.grad_size_bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3636,
  serialized_end=3693,
)


_LINEARMODEL = _descriptor.Descriptor(
  name='LinearModel',
  full_name='innpv.protocol.LinearModel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='slope', full_name='innpv.protocol.LinearModel.slope', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='bias', full_name='innpv.protocol.LinearModel.bias', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3695,
  serialized_end=3737,
)


_HARDWAREINFO = _descriptor.Descriptor(
  name='HardwareInfo',
  full_name='innpv.protocol.HardwareInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='hostname', full_name='innpv.protocol.HardwareInfo.hostname', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='os', full_name='innpv.protocol.HardwareInfo.os', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpus', full_name='innpv.protocol.HardwareInfo.gpus', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3739,
  serialized_end=3797,
)


_UTILIZATIONNODE = _descriptor.Descriptor(
  name='UtilizationNode',
  full_name='innpv.protocol.UtilizationNode',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='slice_id', full_name='innpv.protocol.UtilizationNode.slice_id', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='innpv.protocol.UtilizationNode.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start', full_name='innpv.protocol.UtilizationNode.start', index=2,
      number=3, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='end', full_name='innpv.protocol.UtilizationNode.end', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_forward', full_name='innpv.protocol.UtilizationNode.cpu_forward', index=4,
      number=5, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_forward_span', full_name='innpv.protocol.UtilizationNode.cpu_forward_span', index=5,
      number=6, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_forward', full_name='innpv.protocol.UtilizationNode.gpu_forward', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_forward_span', full_name='innpv.protocol.UtilizationNode.gpu_forward_span', index=7,
      number=8, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_backward', full_name='innpv.protocol.UtilizationNode.cpu_backward', index=8,
      number=9, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cpu_backward_span', full_name='innpv.protocol.UtilizationNode.cpu_backward_span', index=9,
      number=10, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_backward', full_name='innpv.protocol.UtilizationNode.gpu_backward', index=10,
      number=11, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gpu_backward_span', full_name='innpv.protocol.UtilizationNode.gpu_backward_span', index=11,
      number=12, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='children', full_name='innpv.protocol.UtilizationNode.children', index=12,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3800,
  serialized_end=4120,
)


_MEMORYUSAGERESPONSE = _descriptor.Descriptor(
  name='MemoryUsageResponse',
  full_name='innpv.protocol.MemoryUsageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4122,
  serialized_end=4149,
)


_RUNTIMERESPONSE = _descriptor.Descriptor(
  name='RunTimeResponse',
  full_name='innpv.protocol.RunTimeResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4151,
  serialized_end=4174,
)


_ACTIVATIONENTRY = _descriptor.Descriptor(
  name='ActivationEntry',
  full_name='innpv.protocol.ActivationEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4176,
  serialized_end=4199,
)


_WEIGHTENTRY = _descriptor.Descriptor(
  name='WeightEntry',
  full_name='innpv.protocol.WeightEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4201,
  serialized_end=4220,
)


_RUNTIMEENTRY = _descriptor.Descriptor(
  name='RunTimeEntry',
  full_name='innpv.protocol.RunTimeEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4222,
  serialized_end=4242,
)

_FROMCLIENT.fields_by_name['initialize'].message_type = _INITIALIZEREQUEST
_FROMCLIENT.fields_by_name['analysis'].message_type = _ANALYSISREQUEST
_FROMCLIENT.fields_by_name['generic'].message_type = _GENERICEVENT
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['initialize'])
_FROMCLIENT.fields_by_name['initialize'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['analysis'])
_FROMCLIENT.fields_by_name['analysis'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMCLIENT.oneofs_by_name['payload'].fields.append(
  _FROMCLIENT.fields_by_name['generic'])
_FROMCLIENT.fields_by_name['generic'].containing_oneof = _FROMCLIENT.oneofs_by_name['payload']
_FROMSERVER.fields_by_name['error'].message_type = _PROTOCOLERROR
_FROMSERVER.fields_by_name['initialize'].message_type = _INITIALIZERESPONSE
_FROMSERVER.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_FROMSERVER.fields_by_name['throughput'].message_type = _THROUGHPUTRESPONSE
_FROMSERVER.fields_by_name['breakdown'].message_type = _BREAKDOWNRESPONSE
_FROMSERVER.fields_by_name['habitat'].message_type = _HABITATRESPONSE
_FROMSERVER.fields_by_name['energy'].message_type = _ENERGYRESPONSE
_FROMSERVER.fields_by_name['utilization'].message_type = _UTILIZATIONRESPONSE
_FROMSERVER.fields_by_name['ddp'].message_type = _DDPBUCKETSIZESCOMPUTATIONTIMES
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['error'])
_FROMSERVER.fields_by_name['error'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['initialize'])
_FROMSERVER.fields_by_name['initialize'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['analysis_error'])
_FROMSERVER.fields_by_name['analysis_error'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['throughput'])
_FROMSERVER.fields_by_name['throughput'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['breakdown'])
_FROMSERVER.fields_by_name['breakdown'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['habitat'])
_FROMSERVER.fields_by_name['habitat'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['energy'])
_FROMSERVER.fields_by_name['energy'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['utilization'])
_FROMSERVER.fields_by_name['utilization'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_FROMSERVER.oneofs_by_name['payload'].fields.append(
  _FROMSERVER.fields_by_name['ddp'])
_FROMSERVER.fields_by_name['ddp'].containing_oneof = _FROMSERVER.oneofs_by_name['payload']
_HABITATRESPONSE.fields_by_name['predictions'].message_type = _HABITATDEVICEPREDICTION
_HABITATRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_ENERGYRESPONSE.fields_by_name['components'].message_type = _ENERGYCONSUMPTIONCOMPONENT
_ENERGYRESPONSE.fields_by_name['past_measurements'].message_type = _ENERGYRESPONSE
_ENERGYRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_ENERGYCONSUMPTIONCOMPONENT.fields_by_name['component_type'].enum_type = _ENERGYCONSUMPTIONCOMPONENTTYPE
_UTILIZATIONRESPONSE.fields_by_name['rootNode'].message_type = _UTILIZATIONNODE
_UTILIZATIONRESPONSE.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_DDPBUCKETSIZESCOMPUTATIONTIMES.fields_by_name['computation_times'].message_type = _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES
_DDPBUCKETSIZESCOMPUTATIONTIMES.fields_by_name['analysis_error'].message_type = _ANALYSISERROR
_INITIALIZERESPONSE.fields_by_name['entry_point'].message_type = _PATH
_INITIALIZERESPONSE.fields_by_name['hardware'].message_type = _HARDWAREINFO
_ANALYSISERROR.fields_by_name['file_context'].message_type = _FILEREFERENCE
_THROUGHPUTRESPONSE.fields_by_name['run_time_ms'].message_type = _LINEARMODEL
_THROUGHPUTRESPONSE.fields_by_name['peak_usage_bytes'].message_type = _LINEARMODEL
_THROUGHPUTRESPONSE.fields_by_name['batch_size_context'].message_type = _FILEREFERENCE
_BREAKDOWNRESPONSE.fields_by_name['operation_tree'].message_type = _BREAKDOWNNODE
_BREAKDOWNRESPONSE.fields_by_name['weight_tree'].message_type = _BREAKDOWNNODE
_PROTOCOLERROR.fields_by_name['error_code'].enum_type = _PROTOCOLERROR_ERRORCODE
_PROTOCOLERROR_ERRORCODE.containing_type = _PROTOCOLERROR
_FILEREFERENCE.fields_by_name['file_path'].message_type = _PATH
_BREAKDOWNNODE.fields_by_name['contexts'].message_type = _FILEREFERENCE
_BREAKDOWNNODE.fields_by_name['operation'].message_type = _OPERATIONDATA
_BREAKDOWNNODE.fields_by_name['weight'].message_type = _WEIGHTDATA
_BREAKDOWNNODE.oneofs_by_name['data'].fields.append(
  _BREAKDOWNNODE.fields_by_name['operation'])
_BREAKDOWNNODE.fields_by_name['operation'].containing_oneof = _BREAKDOWNNODE.oneofs_by_name['data']
_BREAKDOWNNODE.oneofs_by_name['data'].fields.append(
  _BREAKDOWNNODE.fields_by_name['weight'])
_BREAKDOWNNODE.fields_by_name['weight'].containing_oneof = _BREAKDOWNNODE.oneofs_by_name['data']
_CONTEXTINFO.fields_by_name['context'].message_type = _FILEREFERENCE
_OPERATIONDATA.fields_by_name['context_info_map'].message_type = _CONTEXTINFO
_UTILIZATIONNODE.fields_by_name['children'].message_type = _UTILIZATIONNODE
DESCRIPTOR.message_types_by_name['FromClient'] = _FROMCLIENT
DESCRIPTOR.message_types_by_name['GenericEvent'] = _GENERICEVENT
DESCRIPTOR.message_types_by_name['InitializeRequest'] = _INITIALIZEREQUEST
DESCRIPTOR.message_types_by_name['AnalysisRequest'] = _ANALYSISREQUEST
DESCRIPTOR.message_types_by_name['FromServer'] = _FROMSERVER
DESCRIPTOR.message_types_by_name['HabitatDevicePrediction'] = _HABITATDEVICEPREDICTION
DESCRIPTOR.message_types_by_name['HabitatResponse'] = _HABITATRESPONSE
DESCRIPTOR.message_types_by_name['EnergyResponse'] = _ENERGYRESPONSE
DESCRIPTOR.message_types_by_name['EnergyConsumptionComponent'] = _ENERGYCONSUMPTIONCOMPONENT
DESCRIPTOR.message_types_by_name['UtilizationResponse'] = _UTILIZATIONRESPONSE
DESCRIPTOR.message_types_by_name['DDPBucketSizesComputationTimes'] = _DDPBUCKETSIZESCOMPUTATIONTIMES
DESCRIPTOR.message_types_by_name['DDPExpectedMaximumComputationTimes'] = _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES
DESCRIPTOR.message_types_by_name['InitializeResponse'] = _INITIALIZERESPONSE
DESCRIPTOR.message_types_by_name['AnalysisError'] = _ANALYSISERROR
DESCRIPTOR.message_types_by_name['ThroughputResponse'] = _THROUGHPUTRESPONSE
DESCRIPTOR.message_types_by_name['BreakdownResponse'] = _BREAKDOWNRESPONSE
DESCRIPTOR.message_types_by_name['ProtocolError'] = _PROTOCOLERROR
DESCRIPTOR.message_types_by_name['Path'] = _PATH
DESCRIPTOR.message_types_by_name['FileReference'] = _FILEREFERENCE
DESCRIPTOR.message_types_by_name['BreakdownNode'] = _BREAKDOWNNODE
DESCRIPTOR.message_types_by_name['ContextInfo'] = _CONTEXTINFO
DESCRIPTOR.message_types_by_name['OperationData'] = _OPERATIONDATA
DESCRIPTOR.message_types_by_name['WeightData'] = _WEIGHTDATA
DESCRIPTOR.message_types_by_name['LinearModel'] = _LINEARMODEL
DESCRIPTOR.message_types_by_name['HardwareInfo'] = _HARDWAREINFO
DESCRIPTOR.message_types_by_name['UtilizationNode'] = _UTILIZATIONNODE
DESCRIPTOR.message_types_by_name['MemoryUsageResponse'] = _MEMORYUSAGERESPONSE
DESCRIPTOR.message_types_by_name['RunTimeResponse'] = _RUNTIMERESPONSE
DESCRIPTOR.message_types_by_name['ActivationEntry'] = _ACTIVATIONENTRY
DESCRIPTOR.message_types_by_name['WeightEntry'] = _WEIGHTENTRY
DESCRIPTOR.message_types_by_name['RunTimeEntry'] = _RUNTIMEENTRY
DESCRIPTOR.enum_types_by_name['EnergyConsumptionComponentType'] = _ENERGYCONSUMPTIONCOMPONENTTYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FromClient = _reflection.GeneratedProtocolMessageType('FromClient', (_message.Message,), {
  'DESCRIPTOR' : _FROMCLIENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FromClient)
  })
_sym_db.RegisterMessage(FromClient)

GenericEvent = _reflection.GeneratedProtocolMessageType('GenericEvent', (_message.Message,), {
  'DESCRIPTOR' : _GENERICEVENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.GenericEvent)
  })
_sym_db.RegisterMessage(GenericEvent)

InitializeRequest = _reflection.GeneratedProtocolMessageType('InitializeRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZEREQUEST,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.InitializeRequest)
  })
_sym_db.RegisterMessage(InitializeRequest)

AnalysisRequest = _reflection.GeneratedProtocolMessageType('AnalysisRequest', (_message.Message,), {
  'DESCRIPTOR' : _ANALYSISREQUEST,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.AnalysisRequest)
  })
_sym_db.RegisterMessage(AnalysisRequest)

FromServer = _reflection.GeneratedProtocolMessageType('FromServer', (_message.Message,), {
  'DESCRIPTOR' : _FROMSERVER,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FromServer)
  })
_sym_db.RegisterMessage(FromServer)

HabitatDevicePrediction = _reflection.GeneratedProtocolMessageType('HabitatDevicePrediction', (_message.Message,), {
  'DESCRIPTOR' : _HABITATDEVICEPREDICTION,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HabitatDevicePrediction)
  })
_sym_db.RegisterMessage(HabitatDevicePrediction)

HabitatResponse = _reflection.GeneratedProtocolMessageType('HabitatResponse', (_message.Message,), {
  'DESCRIPTOR' : _HABITATRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HabitatResponse)
  })
_sym_db.RegisterMessage(HabitatResponse)

EnergyResponse = _reflection.GeneratedProtocolMessageType('EnergyResponse', (_message.Message,), {
  'DESCRIPTOR' : _ENERGYRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.EnergyResponse)
  })
_sym_db.RegisterMessage(EnergyResponse)

EnergyConsumptionComponent = _reflection.GeneratedProtocolMessageType('EnergyConsumptionComponent', (_message.Message,), {
  'DESCRIPTOR' : _ENERGYCONSUMPTIONCOMPONENT,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.EnergyConsumptionComponent)
  })
_sym_db.RegisterMessage(EnergyConsumptionComponent)

UtilizationResponse = _reflection.GeneratedProtocolMessageType('UtilizationResponse', (_message.Message,), {
  'DESCRIPTOR' : _UTILIZATIONRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.UtilizationResponse)
  })
_sym_db.RegisterMessage(UtilizationResponse)

DDPBucketSizesComputationTimes = _reflection.GeneratedProtocolMessageType('DDPBucketSizesComputationTimes', (_message.Message,), {
  'DESCRIPTOR' : _DDPBUCKETSIZESCOMPUTATIONTIMES,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.DDPBucketSizesComputationTimes)
  })
_sym_db.RegisterMessage(DDPBucketSizesComputationTimes)

DDPExpectedMaximumComputationTimes = _reflection.GeneratedProtocolMessageType('DDPExpectedMaximumComputationTimes', (_message.Message,), {
  'DESCRIPTOR' : _DDPEXPECTEDMAXIMUMCOMPUTATIONTIMES,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.DDPExpectedMaximumComputationTimes)
  })
_sym_db.RegisterMessage(DDPExpectedMaximumComputationTimes)

InitializeResponse = _reflection.GeneratedProtocolMessageType('InitializeResponse', (_message.Message,), {
  'DESCRIPTOR' : _INITIALIZERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.InitializeResponse)
  })
_sym_db.RegisterMessage(InitializeResponse)

AnalysisError = _reflection.GeneratedProtocolMessageType('AnalysisError', (_message.Message,), {
  'DESCRIPTOR' : _ANALYSISERROR,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.AnalysisError)
  })
_sym_db.RegisterMessage(AnalysisError)

ThroughputResponse = _reflection.GeneratedProtocolMessageType('ThroughputResponse', (_message.Message,), {
  'DESCRIPTOR' : _THROUGHPUTRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ThroughputResponse)
  })
_sym_db.RegisterMessage(ThroughputResponse)

BreakdownResponse = _reflection.GeneratedProtocolMessageType('BreakdownResponse', (_message.Message,), {
  'DESCRIPTOR' : _BREAKDOWNRESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.BreakdownResponse)
  })
_sym_db.RegisterMessage(BreakdownResponse)

ProtocolError = _reflection.GeneratedProtocolMessageType('ProtocolError', (_message.Message,), {
  'DESCRIPTOR' : _PROTOCOLERROR,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ProtocolError)
  })
_sym_db.RegisterMessage(ProtocolError)

Path = _reflection.GeneratedProtocolMessageType('Path', (_message.Message,), {
  'DESCRIPTOR' : _PATH,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.Path)
  })
_sym_db.RegisterMessage(Path)

FileReference = _reflection.GeneratedProtocolMessageType('FileReference', (_message.Message,), {
  'DESCRIPTOR' : _FILEREFERENCE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.FileReference)
  })
_sym_db.RegisterMessage(FileReference)

BreakdownNode = _reflection.GeneratedProtocolMessageType('BreakdownNode', (_message.Message,), {
  'DESCRIPTOR' : _BREAKDOWNNODE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.BreakdownNode)
  })
_sym_db.RegisterMessage(BreakdownNode)

ContextInfo = _reflection.GeneratedProtocolMessageType('ContextInfo', (_message.Message,), {
  'DESCRIPTOR' : _CONTEXTINFO,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ContextInfo)
  })
_sym_db.RegisterMessage(ContextInfo)

OperationData = _reflection.GeneratedProtocolMessageType('OperationData', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONDATA,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.OperationData)
  })
_sym_db.RegisterMessage(OperationData)

WeightData = _reflection.GeneratedProtocolMessageType('WeightData', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTDATA,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.WeightData)
  })
_sym_db.RegisterMessage(WeightData)

LinearModel = _reflection.GeneratedProtocolMessageType('LinearModel', (_message.Message,), {
  'DESCRIPTOR' : _LINEARMODEL,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.LinearModel)
  })
_sym_db.RegisterMessage(LinearModel)

HardwareInfo = _reflection.GeneratedProtocolMessageType('HardwareInfo', (_message.Message,), {
  'DESCRIPTOR' : _HARDWAREINFO,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.HardwareInfo)
  })
_sym_db.RegisterMessage(HardwareInfo)

UtilizationNode = _reflection.GeneratedProtocolMessageType('UtilizationNode', (_message.Message,), {
  'DESCRIPTOR' : _UTILIZATIONNODE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.UtilizationNode)
  })
_sym_db.RegisterMessage(UtilizationNode)

MemoryUsageResponse = _reflection.GeneratedProtocolMessageType('MemoryUsageResponse', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYUSAGERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.MemoryUsageResponse)
  })
_sym_db.RegisterMessage(MemoryUsageResponse)

RunTimeResponse = _reflection.GeneratedProtocolMessageType('RunTimeResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMERESPONSE,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.RunTimeResponse)
  })
_sym_db.RegisterMessage(RunTimeResponse)

ActivationEntry = _reflection.GeneratedProtocolMessageType('ActivationEntry', (_message.Message,), {
  'DESCRIPTOR' : _ACTIVATIONENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.ActivationEntry)
  })
_sym_db.RegisterMessage(ActivationEntry)

WeightEntry = _reflection.GeneratedProtocolMessageType('WeightEntry', (_message.Message,), {
  'DESCRIPTOR' : _WEIGHTENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.WeightEntry)
  })
_sym_db.RegisterMessage(WeightEntry)

RunTimeEntry = _reflection.GeneratedProtocolMessageType('RunTimeEntry', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMEENTRY,
  '__module__' : 'innpv_pb2'
  # @@protoc_insertion_point(class_scope:innpv.protocol.RunTimeEntry)
  })
_sym_db.RegisterMessage(RunTimeEntry)


# @@protoc_insertion_point(module_scope)





class OperationInfo:
    def __init__(self, bound_name, op_name, ast_node, position, perf_hints):
        self.bound_name = bound_name
        self.op_name = op_name
        self.ast_node = ast_node
        self.position = position
        self.perf_hints = perf_hints
        self.usages = []
        self.runtime_us = 0

    def set_usages(self, usages):
        self.usages = usages

    def add_to_runtime_us(self, runtime_us):
        self.runtime_us += runtime_us


class OperationInfoMap:
    def __init__(self):
        self.operations = {}

    def add_operation_info(self, operation):
        self.operations[operation.bound_name] = operation

    def get_operation_info_by_bound_name(self, bound_name):
        if bound_name not in self.operations:
            return None
        return self.operations[bound_name]

    def get_operations(self):
        return self.operations.values()

    def set_runtimes_from_cache(self, cached_info_map):
        """
        Used to set the runtimes from cache for when the parsed code has not
        changed.
        """
        for bound_name, op_info in self.operations.items():
            cached_op_info = cached_info_map.get_operation_info_by_bound_name(
                bound_name)
            op_info.runtime_us = cached_op_info.runtime_us


class AnnotationInfo:
    def __init__(self, input_size, start_position, end_position):
        self.input_size = input_size
        self.start_position = start_position
        self.end_position = end_position


class PerformanceHint:
    def __init__(self, keyword, position, effectiveness, natural_direction):
        self.keyword = keyword
        self.position = position
        self.effectiveness = effectiveness
        self.natural_direction = natural_direction


class LinearModel:
    def __init__(self, coefficient, bias):
        self.coefficient = coefficient
        self.bias = bias

    def __repr__(self):
        return 'LinearModel(coefficient={:.4f}, bias={:.4f})'.format(
            self.coefficient, self.bias)

    def evaluate(self, x):
        return self.coefficient * x + self.bias

    def inverse(self, y):
        return (y - self.bias) / self.coefficient


class MemoryInfo:
    def __init__(self, usage_model_mb, usage_mb, max_capacity_mb):
        self.usage_model_mb = usage_model_mb
        self.usage_mb = usage_mb
        self.max_capacity_mb = max_capacity_mb

    def __repr__(self):
        return 'MemoryInfo(model={}, usage_mb={}, capacity_mb={})'.format(
            self.usage_model_mb, self.usage_mb, self.max_capacity_mb)


class ThroughputInfo:
    def __init__(
        self,
        throughput,
        max_throughput,
        runtime_model_ms
    ):
        self.throughput = throughput
        self.max_throughput = max_throughput
        self.runtime_model_ms = runtime_model_ms

    def __repr__(self):
        return (
            'ThroughputInfo(thpt={}, max_thpt={}, model={})'
            .format(
                self.throughput,
                self.max_throughput,
                self.runtime_model_ms,
            )
        )

    def batch_from_throughput(self, throughput):
        # Thpt = batch / runtime_model
        throughput_ms = throughput / 1000
        return (
            (throughput_ms * self.runtime_model_ms.bias) /
            (1 - throughput_ms * self.runtime_model_ms.coefficient)
        )


class PerformanceLimits:
    def __init__(self, max_batch_size, throughput_limit):
        self.max_batch_size = max_batch_size
        self.throughput_limit = throughput_limit

    def __repr__(self):
        return (
            'PerformanceLimits(max_batch={:.2f}, thpt_limit={:.2f})'.format(
                self.max_batch_size,
                self.throughput_limit,
            )
        )



import time
from threading import Thread
import numpy as np

import pynvml as N
from pyRAPL import Sensor

class CPUMeasurer:
    def __init__(self, interval):
        self.interval = interval
        self.power = []
        self.last_cpu = None
        self.last_dram = None

    def measurer_init(self):
        self.sensor = None
        try:
            self.sensor = Sensor()
            energy = self.sensor.energy()
            self.last_cpu = np.array(energy[0::2])
            self.last_dram = np.array(energy[1::2])
        except Exception:
            print("Warning. Failed to get CPU energy. \
                  You need to set the right permissions for pyRAPL")
            print("eg. $ sudo chmod -R a+r /sys/class/powercap/intel-rapl")

    def measurer_measure(self):
        # Get energy consumed so far (since last CPU reset)
        if self.sensor is None:
            return

        energy = self.sensor.energy()
        cpu = np.array(energy[0::2])
        dram = np.array(energy[1::2])

        # Compare against last measurement to determine energy since last measure
        diff_cpu = cpu - self.last_cpu
        dram - self.last_dram

        # 1J = 10^6 uJ
        # The cpu used this much since the last measurement
        # We have mW = 1000*J/s = 1000*(uJ/10^6)/s
        cpu_total = np.sum(diff_cpu)
        cpu_mW = 1000 * (cpu_total / 1e6) / self.interval
        self.power.append(cpu_mW)

        self.last_cpu = cpu
        self.last_dram = dram

    def measurer_deallocate(self):
        pass

    def total_energy(self):
        if len(self.power) == 0:
            return None

        # J = W * s,    1W = 1000 mW
        energy = self.interval * sum(self.power) / 1000.0
        return energy

class GPUMeasurer:
    def __init__(self, interval):
        self.interval = interval
        self.power = []

    def measurer_init(self):
        N.nvmlInit()
        self.device_handle = N.nvmlDeviceGetHandleByIndex(0)

    def measurer_measure(self):
        power = N.nvmlDeviceGetPowerUsage(self.device_handle)
        self.power.append(power)

    def measurer_deallocate(self):
        N.nvmlShutdown()

    def total_energy(self):
        # J = W * s,    1W = 1000 mW
        energy = self.interval * sum(self.power) / 1000.0
        return energy

class EnergyMeasurer:
    def __init__(self):
        self.sleep_interval = 0.1
        self.measuring = False
        self.measure_thread = None

        self.measurers = {
            "cpu": CPUMeasurer(self.sleep_interval),
            "gpu": GPUMeasurer(self.sleep_interval),
        }

    def run_measure(self):
        # Initialize
        for m in self.measurers:
            self.measurers[m].measurer_init()

        # Run measurement loop
        while self.measuring:
            for m in self.measurers:
                self.measurers[m].measurer_measure()
            time.sleep(self.sleep_interval)

        # Cleanup
        for m in self.measurers:
            self.measurers[m].measurer_deallocate()

    def begin_measurement(self):
        assert(self.measure_thread is None)
        self.measure_thread = Thread(target=self.run_measure)
        self.measuring = True
        self.measure_thread.start()

    def end_measurement(self):
        self.measuring = False
        self.measure_thread.join()
        self.measure_thread = None

    def total_energy(self):
        total_energy = 0.
        for m in self.measurers:
            e = self.measurers[m].total_energy()
            if e is not None:
                total_energy += e
        return total_energy

    def cpu_energy(self):
        return self.measurers["cpu"].total_energy()

    def gpu_energy(self):
        return self.measurers["gpu"].total_energy()



import argparse
import sys

import deepview_profile.commands.measurements
import deepview_profile.commands.prediction_models


def main():
    parser = argparse.ArgumentParser(
        prog="deepview-evaluate",
        description="DeepView Evaluation Tool",
    )
    subparsers = parser.add_subparsers(title="Commands")
    deepview_profile.commands.measurements.register_command(subparsers)
    deepview_profile.commands.prediction_models.register_command(subparsers)
    args = parser.parse_args()

    if 'func' not in args:
        parser.print_help()
        sys.exit(1)

    # Run the specified command
    args.func(args)


if __name__ == '__main__':
    main()



import pynvml


class NVML:
    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.stop()

    def start(self):
        pynvml.nvmlInit()

    def stop(self):
        pynvml.nvmlShutdown()

    def get_memory_capacity(self):
        # TODO: Support multiple devices
        handle = pynvml.nvmlDeviceGetHandleByIndex(0)
        return pynvml.nvmlDeviceGetMemoryInfo(handle)

    def get_device_names(self):
        device_names = []
        for i in range(pynvml.nvmlDeviceGetCount()):
            handle = pynvml.nvmlDeviceGetHandleByIndex(i)
            device_name = pynvml.nvmlDeviceGetName(handle).decode("utf-8")
            device_names.append(device_name)
        return device_names
    
